using System;
using System.IO;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <see cref="IStatsReporting"/> interface defines the contract to aggregate and
    /// render the statistical report. Result is fed into this interface one by one and
    /// the file report is then generated. There is a way to reset (clear) the current
    /// aggregated results.
    ///
    /// <contract-constructor>
    /// All implementation should have a constructor with a Property as an argument
    /// in order to reflect from configuration file. This requirement can be dropped
    /// if it's not necessary to use this way.
    /// </contract-constructor>
    ///
    /// <thread-safety>
    /// There is no thread-safety requirement/recommendation for the implementations.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public interface IStatsReporting
    {
        /// <summary>
        /// Aggregate the result into the reporting implementation.
        /// The results are kept by the reporting itself.
        /// </summary>
        ///
        /// <param name="processor">the processor that generated the result</param>
        /// <param name="file">the file that is processed</param>
        /// <param name="result">the processing result</param>
        ///
        /// <exception cref="StatsReportingException">if file is not in base dir</exception>
        /// <exception cref="ArgumentNullException">if any argument is null</exception>
        void Aggregate(IFileProcessor processor, FileInfo file, StatsResult result);

        /// <summary>
        /// Generate the report with all the aggregated results.
        /// </summary>
        ///
        /// <returns>the generated report</returns>
        ///
        /// <exception cref="StatsReportingException">if error occurs</exception>
        string GenerateReport();

        /// <summary>
        /// Clear all the aggregated result and set the base dir (which is nullable).
        /// </summary>
        ///
        /// <param name="baseDirectory">the base dir to reset</param>
        void Reset(DirectoryInfo baseDirectory);
    }
}