using System;
using System.Collections;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileNameMatcher"/> concrete implementation that matches one or more file extensions.
    /// The extensions are matched in a case-insensitive manner.
    ///
    /// <configuration>
    /// This class can be constructed via reflection by the constructor contract of
    /// <see cref="IFileMatcher"/>.
    /// The supported property is "extensions" which is potentially multi-valued.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class ExtensionMatcher : FileNameMatcher
    {
        /// <summary>
        /// Configuration key that contains multiple file extensions to match.
        /// Though extensions are saved as they are the matching is case-insensitive.
        /// Empty is allowed in order to match a file with no extension.
        /// </summary>
        public static readonly string ExtensionsPropertyKey = "extensions";

        /// <summary>
        /// A list of file extensions (String) to match.
        /// This list is initialized in constructor and is immutable.
        /// </summary>
        private IList extensions = new ArrayList();

        /// <summary>
        /// Constructor with single extension. The extension can be empty.
        /// </summary>
        ///
        /// <param name="extension">a single file extension</param>
        /// <exception cref="ArgumentNullException">if extension is null</exception>
        public ExtensionMatcher(string extension)
        {
            // the extension can be empty string
            FileStatisticsHelper.CheckNull(extension, "exstension");

            if (extension.Trim().Length == 0)
            {
                // if extension is empty string use string.Empty instead
                // for windows file is not allowed with empty string as extension
                extensions.Add(string.Empty);
            }
            else
            {
                // add the extension
                extensions.Add(extension.ToLower());
            }
        }

        /// <summary>
        /// Constructor with multiple extensions. Simply save a clone of the list.
        /// </summary>
        ///
        /// <param name="extensions">the extensions to match</param>
        ///
        /// <exception cref="ArgumentNullException">if extensions is null</exception>
        /// <exception cref="ArgumentException">
        /// if extensions is empty or contains non-string instance</exception>
        public ExtensionMatcher(IList extensions)
        {
            // verify the extensions is valid
            FileStatisticsHelper.CheckNull(extensions, "extensions");

            // save the types
            FileStatisticsHelper.SaveListValue(this.extensions, extensions, "extensions", true);
        }

        /// <summary>
        /// Constructor that loads extensions from a property.
        /// It should look for an "extensions" sub-property and extract multiple values.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if property does not contain
        /// an extensions sub-property or it does not hold any values</exception>
        public ExtensionMatcher(string nameSpace, string prefix)
        {
            // verify arguments
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            // get the property name
            string propertyName = (prefix.Trim().Length == 0) ?
                ExtensionsPropertyKey : prefix + "." + ExtensionsPropertyKey;

            // load the config values
            FileStatisticsHelper.LoadConfigValues(nameSpace, propertyName, this.extensions, true);
        }

        /// <summary>
        /// Checks whether the filename matches any of the extensions. The extension is extracted after
        /// the last dot in the filename. If the filename does not contain a dot the extension is empty
        /// Matching should be case-insensitive.
        /// </summary>
        ///
        /// <param name="filename">the filename to match</param>
        /// <returns>whether the filename matches</returns>
        ///
        /// <exception cref="FileMatchingException">if error occurs</exception>
        protected override bool Match(string filename)
        {
            return FileStatisticsHelper.Match(this.extensions, filename.ToLower());
        }
    }
}