using System;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// This is a private inner class to <see cref="CStyleLineCounter"/> that implements
    /// a state machine that can process the source code.
    ///
    /// Notice this is an implementation recommendation. Developer can choose to improve
    /// from this start point. Dropping this class is acceptable as long as the implementation
    /// of <see cref="CStyleLineCounter"/> is stateless (and the clarity and/or efficiency
    /// aspects are improved).
    ///
    /// <thread-safety>
    /// This class is stateful (by name) and not thread-safety.
    /// Each instance only serves a single process.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    internal class SimpleStateMachine
    {
        /// <summary>
        /// Represents the state not in comment or literal.
        /// </summary>
        private const int StateStart = 1;

        /// <summary>
        /// Represents the state after a single slash.
        /// (Input on slash would enter line comment state; input on star would enter block state.)
        /// </summary>
        private const int StateSlash = 2;

        /// <summary>
        /// Represents the state of a star in a block comment.
        /// (Input of slash would enter start stat).
        /// </summary>
        private const int StateStar = 3;

        /// <summary>
        /// Represents the state of back slash in single quote literal (next character is escaped).
        /// </summary>
        private const int StateSQBackslash = 4;

        /// <summary>
        /// Represents the state of back slash in double quote literal (next character is escaped).
        /// </summary>
        private const int StateDQBackslash = 5;

        /// <summary>
        /// Represents the state in single quote literal.
        /// </summary>
        private const int StateSingleQuote = 6;

        /// <summary>
        /// Represents the state in double quote literal.
        /// </summary>
        private const int StateDoubleQuote = 7;

        /// <summary>
        /// Represents the state in block comment.
        /// </summary>
        private const int StateBlockComment = 8;

        /// <summary>
        /// Represents the state in line comment.
        /// </summary>
        private const int StateLineComment = 9;

        /// <summary>
        /// Represents the slash character.
        /// </summary>
        private const char CHAR_SLASH = '/';

        /// <summary>
        /// Represents the comma character.
        /// </summary>
        private const char CHAR_COMMA = ';';

        /// <summary>
        /// Represents the open brace character.
        /// </summary>
        private const char CHAR_OPEN_BRACE = '{';

        /// <summary>
        /// Represents the star character.
        /// </summary>
        private const char CHAR_STAR = '*';

        /// <summary>
        /// Represents the back slash character.
        /// </summary>
        private const char CHAR_BACK_SLASH = '\\';

        /// <summary>
        /// Represents the single quote character
        /// </summary>
        private const char CHAR_SINGLE_QUOTE = '\'';

        /// <summary>
        /// Represents the double quote charater
        /// </summary>
        private const char CHAR_DOUBLE_QUOTE = '\"';

        /// <summary>
        /// Represents the line feed character.
        /// </summary>
        private const char CHAR_LINE_FEED = (char) 0xa;

        /// <summary>
        /// The state of this state machine. Reference the consts.
        /// </summary>
        private int state = StateStart;

        /// <summary>
        /// The current line count. Initialized to 0 and increments whenever necessary.
        /// </summary>
        private long lines = 0;

        /// <summary>
        /// Get the line count so far. Simply return the variable.
        /// </summary>
        /// <value>Represents the line count so far.</value>
        public long Lines
        {
            get
            {
                return lines;
            }
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        public SimpleStateMachine()
        {
        }

        /// <summary>
        /// Process a line. It processes each character as well as a line feed.
        /// </summary>
        /// <param name="line">the line to process</param>
        ///
        /// <exception cref="ArgumentNullException">if line is null</exception>
        public void Process(string line)
        {
            FileStatisticsHelper.CheckNull(line, "line");

            int length = line.Length;
            for (int i = 0; i < length; ++i)
            {
                Process(line[i]);
            }
        }

        /// <summary>
        /// Process a character from input. Refer to CS for algorithm.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void Process(char ch)
        {
            switch (state)
            {
                case StateStart:
                    ProcessStart(ch);
                    break;
                case StateSlash:
                    ProcessSlash(ch);
                    break;
                case StateStar:
                    ProcessStar(ch);
                    break;
                case StateSQBackslash:
                    ProcessSQBackslash(ch);
                    break;
                case StateDQBackslash:
                    ProcessDQBackslash(ch);
                    break;
                case StateSingleQuote:
                    ProcessSingleQuote(ch);
                    break;
                case StateDoubleQuote:
                    ProcessDoubleQuote(ch);
                    break;
                case StateBlockComment:
                    ProcessBlockComment(ch);
                    break;
                case StateLineComment:
                    ProcessLineComment(ch);
                    break;
            }
        }

        /// <summary>
        /// Process the character input when the state is Start.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessStart(char ch)
        {
            if (ch == CHAR_SLASH)
            {
                state = StateSlash;
            }
            else if ((ch == CHAR_COMMA) || (ch == CHAR_OPEN_BRACE))
            {
                ++lines;
            }
            else if (ch == CHAR_DOUBLE_QUOTE)
            {
                state = StateDoubleQuote;
            }
            else if (ch == CHAR_SINGLE_QUOTE)
            {
                state = StateSingleQuote;
            }
        }

        /// <summary>
        /// Process the character input when the state is Slash.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessSlash(char ch)
        {
            if (ch == CHAR_STAR)
            {
                state = StateBlockComment;
            }
            else if (ch == CHAR_SLASH)
            {
                state = StateLineComment;
            }
            else
            {
                state = StateStart;
            }
        }

        /// <summary>
        /// Process the character input when the state is Star.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessStar(char ch)
        {
            if (ch == CHAR_SLASH)
            {
                state = StateStart;
            }
            else if (ch == CHAR_STAR)
            {
                state = StateStar;
            }
            else
            {
                state = StateBlockComment;
            }
        }

        /// <summary>
        /// Process the character input when the state is SQBackslash.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessSQBackslash(char ch)
        {
            state = StateSingleQuote;
        }

        /// <summary>
        /// Process the character input when the state is DQBackslash.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessDQBackslash(char ch)
        {
            state = StateDoubleQuote;
        }

        /// <summary>
        /// Process the character input when the state is SingleQuote.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessSingleQuote(char ch)
        {
            if (ch == CHAR_BACK_SLASH)
            {
                state = StateSQBackslash;
            }
            else if (ch == CHAR_SINGLE_QUOTE)
            {
                state = StateStart;
            }
        }

        /// <summary>
        /// Process the character input when the state is DoubleQuote.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessDoubleQuote(char ch)
        {
            if (ch == CHAR_BACK_SLASH)
            {
                state = StateDQBackslash;
            }
            else if (ch == CHAR_DOUBLE_QUOTE)
            {
                state = StateStart;
            }
        }

        /// <summary>
        /// Process the character input when the state is BlockComment.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessBlockComment(char ch)
        {
            if (ch == CHAR_STAR)
            {
                state = StateStart;
            }
        }

        /// <summary>
        /// Process the character input when the state is LineComment.
        /// </summary>
        ///
        /// <param name="ch">the character to process</param>
        private void ProcessLineComment(char ch)
        {
            if (ch == CHAR_LINE_FEED)
            {
                state = StateStart;
            }
        }
    }
}