using System;
using System.IO;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// This class provides a <see cref="IStatsReporting"/> implementation base.
    /// It aggregates results from each processor and classifies them based on
    /// single file, directory or alias. It also provides an overall stats.
    /// It defers the actual rendering logic to concrete implementations
    /// (<see cref="BasicStatsReporting"/> and <see cref="XmlStatsReporting"/>).
    ///
    /// <configuration>
    /// This class has a constructor with <see cref="Property"/> for extension purpose.
    /// There is no real logic in this release.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is not thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public abstract class AbstractStatsReporting : IStatsReporting
    {
        /// <summary>
        /// Represents the directory seperator.
        /// </summary>
        private const char SEPERATOR = '\\';

        /// <summary>
        /// The stats mapped to each file. The file path is relative to base dir or absolute
        /// if no base dir. Its reference never changes.
        /// </summary>
        private StatsCollection fileStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each directory. The dir path is relative to base dir or absolute
        /// if no base dir. Its reference never changes.
        /// </summary>
        private StatsCollection directoryStats = new StatsCollection();

        /// <summary>
        /// The stats mapped to each alias.
        /// </summary>
        private StatsCollection aliasStats = new StatsCollection();

        /// <summary>
        /// The overall stats. It is created a new when reporting resets.
        /// </summary>
        private StatsResult overallStats = new StatsResult();

        /// <summary>
        /// The base dir for the reporting. This variable is nullable
        /// (meaning all paths are absolute).
        /// </summary>
        private DirectoryInfo baseDirectory = null;

        /// <summary>
        /// Get the file stats.
        /// </summary>
        ///
        /// <value>Represents the file stats.</value>
        public StatsCollection FileStats
        {
            get
            {
                return this.fileStats;
            }
        }

        /// <summary>
        /// Get the dir stats.
        /// </summary>
        ///
        /// <value>Represents the dir stats.</value>
        public StatsCollection DirectoryStats
        {
            get
            {
                // Simply return the variable.
                return this.directoryStats;
            }
        }

        /// <summary>
        /// Get the alias stats.
        /// </summary>
        ///
        /// <value>Represents the alias stats.</value>
        public StatsCollection AliasStats
        {
            get
            {
                // Simply return the variable.
                return this.aliasStats;
            }
        }

        /// <summary>
        /// Get the overall stats.
        /// </summary>
        ///
        /// <value>Represents the base dir.</value>
        public StatsResult OverallStats
        {
            get
            {
                // Return a clone.
                return new StatsResult(this.overallStats);
            }
        }

        /// <summary>
        /// Get the base dir.
        /// </summary>
        ///
        /// <value>Represents the base dir.</value>
        public DirectoryInfo BaseDirectory
        {
            get
            {
                // Simply return the variable.
                return this.baseDirectory;
            }
        }

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected AbstractStatsReporting()
        {
        }

        /// <summary>
        /// Constructor with property. This does not have any functionality in the current release
        /// and is kept for extension purpose. Please also declare the exceptions.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        protected AbstractStatsReporting(string nameSpace, string prefix)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");
        }

        /// <summary>
        /// Aggregate a result into the reporting.
        /// </summary>
        ///
        /// <param name="processor">the processor that generated the result</param>
        /// <param name="file">the file that is processed</param>
        /// <param name="result">the result</param>
        ///
        /// <exception cref="ArgumentNullException">if processor or file or result is null.</exception>
        /// <exception cref="StatsReportingException">if file is not in base dir.</exception>
        public void Aggregate(IFileProcessor processor, FileInfo file, StatsResult result)
        {
            FileStatisticsHelper.CheckNull(processor, "processor");
            FileStatisticsHelper.CheckNull(file, "file");
            FileStatisticsHelper.CheckNull(result, "result");

            // Get the file name. If base dir exists, strip the base portion off.
            string fileName = file.FullName;
            if (this.BaseDirectory != null)
            {
                if (fileName.ToLower().Equals(this.baseDirectory.FullName.ToLower()))
                {
                    throw new StatsReportingException("File is same as the base directory");
                }
                if (fileName.IndexOf(this.BaseDirectory.FullName) < 0)
                {
                    throw new StatsReportingException("The file '" + fileName
                        + "' is not in the base dir '" + this.BaseDirectory.FullName + "'");
                }
                fileName = fileName.Substring(this.BaseDirectory.FullName.Length + 1);
            }

            // Aggregate the result against file, each path it is in, alias and the overall stats.
            // add file stattistics
            this.fileStats.Add(fileName, result);

            // add directory statistics
            AddDirectoryStats(fileName, result);

            // add alias statistics
            this.aliasStats.Add(processor.Alias, result);

            // add overall
            this.overallStats.Aggregate(result);
        }

        /// <summary>
        /// Aggregate the result against each directory.
        /// </summary>
        ///
        /// <param name="fileName">the file that is processed.</param>
        /// <param name="result">the result</param>
        private void AddDirectoryStats(string fileName, StatsResult result)
        {
            string[] dirs = fileName.Split(new char[] {SEPERATOR});
            if (dirs.Length == 1)
            {
                // in base dir not add to directory stats
            }
            else
            {
                string dir = "";
                for (int i = 0; i < dirs.Length - 1; ++i)
                {
                    dir += dirs[i];
                    this.directoryStats.Add(dir, result);
                    dir += SEPERATOR;
                }
            }
        }

        /// <summary>
        /// Reset the reporting. Base dir can be null when absolute paths are desirable.
        /// </summary>
        ///
        /// <param name="baseDirectory">the base dir to use</param>
        ///
        /// <exception cref="ArgumentNullException">if baseDirectory is null</exception>
        /// <exception cref="ArgumentException">if baseDirectory is not a valid dir</exception>
        public void Reset(DirectoryInfo baseDirectory)
        {
            // verify the base directory is valid
            FileStatisticsHelper.CheckNull(baseDirectory, "baseDirectory");
            if (!baseDirectory.Exists)
            {
                throw new ArgumentException("The baseDirectory is not a valid dir.", "baseDirectory");
            }

            // Clear all the collections.
            this.fileStats.Clear();
            this.directoryStats.Clear();
            this.aliasStats.Clear();

            // Create a new overall StatsResult
            this.overallStats = new StatsResult();

            // Set the base dir.
            this.baseDirectory = baseDirectory;
        }

        /// <summary>
        /// Generate the report with all the aggregated results.
        /// </summary>
        ///
        /// <returns>the generated report</returns>
        public abstract string GenerateReport();
    }
}