using System;
using System.Text;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// <see cref="BasicStatsReporting"/> is a reporting implementation that
    /// renders user-friendly reports.
    ///
    /// <configuration>
    /// It does not support any configuration.
    /// The constructor is included to address reflection contract.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is not thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class BasicStatsReporting : AbstractStatsReporting
    {
        /// <summary>
        /// Represents the report tile.
        /// </summary>
        private const string REPORT_TITLE = "--------------File Statistics Report--------------\r\n";

        /// <summary>
        /// Indicates the metric content formatter.
        /// </summary>
        private const string METRIC_CONTENT = "    {0}:{1}\r\n";

        /// <summary>
        /// Represents the filestats begin tip.
        /// </summary>
        private const string FILESTATS_TILTE_BEGIN = "<<<File Statistics\r\n";

        /// <summary>
        /// Indicates the format of the file title.
        /// </summary>
        private const string FILE_TITLE = "File:{0}\r\n";

        /// <summary>
        /// Represents the filestats end tip.
        /// </summary>
        private const string FILESTATS_TILTE_END = "File Statistics>>>\r\n\r\n";

        /// <summary>
        /// Represents the directorystats begin tip.
        /// </summary>
        private const string DIRECTORYSTATS_TILTE_BEGIN = "<<<Directory Statistics\r\n";

        /// <summary>
        /// Indicates the format of the directory title.
        /// </summary>
        private const string DIRECTORY_TITLE = "Directory:{0}\r\n";

        /// <summary>
        /// Represents the directorystats end tip.
        /// </summary>
        private const string DIRECTORYSTATS_TILTE_END = "Directory Statistics>>>\r\n\r\n";

        /// <summary>
        /// Represents the aliasstats begin tip.
        /// </summary>
        private const string ALIASSTATS_TILTE_BEGIN = "<<<Aliase Statistics\r\n";

        /// <summary>
        /// Indicates the format of the alias title.
        /// </summary>
        private const string ALIAS_TITLE = "Alias:{0}\r\n";

        /// <summary>
        /// Represents the aliasstats end tip.
        /// </summary>
        private const string ALIASSTATS_TILTE_END = "Alias Statistics>>>\r\n\r\n";

        /// <summary>
        /// Represents the overallstats begin tip.
        /// </summary>
        private const string OVERALLSTATS_TILTE_BEGIN = "<<<Overall Statistics\r\n";

        /// <summary>
        /// /// Represents the overallstats end tip.
        /// </summary>
        private const string OVERALLSTATS_TILTE_END = "Overall Statistics>>>";

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public BasicStatsReporting()
        {
        }

        /// <summary>
        ///  Constructor that loads config from the nameSpace.
        /// </summary>
        ///
        /// <param name="nameSpace">non-empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        public BasicStatsReporting(string nameSpace, string prefix) : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// This generates a user-readable report.
        /// Include info for everything hold on the reporting in reasonable format.
        /// </summary>
        ///
        /// <returns>the generated report</returns>
        ///
        /// <exception cref="StatsReportingException">if rendering fails</exception>
        public override string GenerateReport()
        {
            StringBuilder buffer = new StringBuilder();
            buffer.Append(REPORT_TITLE);

            GenerateFiles(buffer);
            GenerateDirectories(buffer);
            GenerateAliases(buffer);
            GenerateOverall(buffer);

            return buffer.ToString();
        }

        /// <summary>
        /// Output the filestats to buffer.
        /// </summary>
        ///
        /// <param name="buffer">the buffer to hold the report.</param>
        private void GenerateFiles(StringBuilder buffer)
        {
            StatsResult result = null;

            buffer.Append(FILESTATS_TILTE_BEGIN);
            foreach (string key in this.FileStats.AllKeys)
            {
                buffer.AppendFormat(FILE_TITLE, key);
                result = this.FileStats[key];
                foreach (Metric metric in result.Metrics)
                {
                    buffer.AppendFormat(METRIC_CONTENT, metric.DisplayName, result[metric]);
                }
            }
            buffer.Append(FILESTATS_TILTE_END);
        }

        /// <summary>
        /// Output the directorystats to buffer.
        /// </summary>
        ///
        /// <param name="buffer">the buffer to hold the report.</param>
        private void GenerateDirectories(StringBuilder buffer)
        {
            StatsResult result = null;

            buffer.Append(DIRECTORYSTATS_TILTE_BEGIN);
            foreach (string key in this.DirectoryStats.AllKeys)
            {
                buffer.AppendFormat(DIRECTORY_TITLE, key);
                result = this.DirectoryStats[key];
                foreach (Metric metric in result.Metrics)
                {
                    buffer.AppendFormat(METRIC_CONTENT, metric.DisplayName, result[metric]);
                }
            }
            buffer.Append(DIRECTORYSTATS_TILTE_END);
        }

        /// <summary>
        /// Output the aliasstats to buffer.
        /// </summary>
        ///
        /// <param name="buffer">the buffer to hold the report.</param>
        private void GenerateAliases(StringBuilder buffer)
        {
            StatsResult result = null;

            buffer.Append(ALIASSTATS_TILTE_BEGIN);
            foreach (string key in this.AliasStats.AllKeys)
            {
                buffer.AppendFormat(ALIAS_TITLE, key);
                result = this.AliasStats[key];
                foreach (Metric metric in result.Metrics)
                {
                    buffer.AppendFormat(METRIC_CONTENT, metric.DisplayName, result[metric]);
                }
            }
            buffer.Append(ALIASSTATS_TILTE_END);
        }

        /// <summary>
        /// Output the overallstats to buffer.
        /// </summary>
        ///
        /// <param name="buffer">the buffer to hold the report.</param>
        private void GenerateOverall(StringBuilder buffer)
        {
            buffer.Append(OVERALLSTATS_TILTE_BEGIN);

            StatsResult result = this.OverallStats;
            foreach (Metric metric in result.Metrics)
            {
                buffer.AppendFormat(METRIC_CONTENT, metric.DisplayName, result[metric]);
            }
            buffer.Append(OVERALLSTATS_TILTE_END);
        }
    }
}