using System;
using System.IO;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// Unit test for <see cref="AbstractStatsReporting"/> class.
    /// It mainly tests the following:
    /// constructor: AbstractStatsReporting(), AbstractStatsReporting(string, string)
    /// property: FileStats, DirectoryStats, AliasStats, OverallStats, BaseDirectory
    /// member: Aggregate(IFileProcessor, FileInfo, StatsResult), Reset(DirectoryInfo),
    /// AddDirectoryStats(string, StatsResult)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class AbstractStatsReportingUnitTest
    {
        /// <summary>
        /// Represent the instance of <see cref="IFileProcessor"/> implementation,
        /// used to construct the report.
        /// </summary>
        private IFileProcessor processor = new TextLineCounter("Text", new AnyFileMatcher());

        /// <summary>
        /// Represents the instance of FileInfo, used to construct the report.
        /// </summary>
        private FileInfo file = new FileInfo("../../test_files/processor/lineCounter/text1.txt");

        /// <summary>
        /// Represents the instance of <see cref="StatsResult"/>, used to construct the report.
        /// </summary>
        private StatsResult result = null;

        /// <summary>
        /// Represents an instance of <see cref="AbstractStatsReporting"/>, used to test it public member.
        /// </summary>
        private AbstractStatsReporting report;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);

            result = new StatsResult();
            Metric metric = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result[metric] = 1;

            report = new MockStatsReporting(UnitTestHelper.DEFAULT_NAMESPACE, "reporting");
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
        }

        /// <summary>
        /// Test ctor AbstractStatsReporting(), instanc should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new MockStatsReporting(),
                "Failed to create the instance of AbstractStatsReporting.");
        }

        /// <summary>
        /// Test ctor AbstractStatsReporting(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NamespaceIsNull()
        {
            new MockStatsReporting(null, "reporting");
        }

        /// <summary>
        /// Test ctor AbstractStatsReporting(string nameSpace, string prefix), with nameSpace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NamespaceIsEmpty()
        {
            new MockStatsReporting("  ", "reporting");
        }

        /// <summary>
        /// Test ctor AbstractStatsReporting(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PrefixIsNull()
        {
            new MockStatsReporting("topcoder", null);
        }

        /// <summary>
        /// Test ctor AbstractStatsReporting(string nameSpace, string prefix), with nameSpace is empty string,
        /// AE is expected.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Assert.IsNotNull(report, "Failed to create instance of AbstractStatsReporting.");
        }

        /// <summary>
        /// Test Reset(DirectoryInfo baseDirectory), with baseDirectory is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestReset_BaseDirectoryIsNull()
        {
            report.Reset(null);
        }

        /// <summary>
        /// Test Reset(DirectoryInfo baseDirectory), with baseDirectory is not valid dir,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestReset_BaseDirectoryIsInvalid()
        {
            report.Reset(new DirectoryInfo("../../test_files/NonExisted"));
        }

        /// <summary>
        /// Test Reset(DirectoryInfo baseDirectory), with baseDirectory is valid dir,
        /// collections should be clear, result should be re-construct.
        /// </summary>
        [Test]
        public void TestReset()
        {
            DirectoryInfo dir = new DirectoryInfo("../../test_files");
            report.Reset(dir);
            Assert.AreEqual(dir, report.BaseDirectory,
                "Failed to reset the report, the base directory not set correctly.");
            Assert.AreEqual(0, report.FileStats.AllKeys.Count,
                "Failed to reset the report, the FileStats not set correctly.");
            Assert.AreEqual(0, report.DirectoryStats.AllKeys.Count,
                "Failed to reset the report, the DirectoryStats not set correctly.");
            Assert.AreEqual(0, report.AliasStats.AllKeys.Count,
                "Failed to reset the report, the AliasStats not set correctly.");
            Assert.AreEqual(0, report.OverallStats.Metrics.Count,
                "Failed to reset the report, the OverallStats not set correctly.");
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// with processor is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregate_ProcessorIsNull()
        {
            report.Aggregate(null, file, result);
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// with file is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregate_FileIsNull()
        {
            report.Aggregate(processor, null, result);
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// with result is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregate_ResultIsNull()
        {
            report.Aggregate(processor, file, null);
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// with result is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(StatsReportingException))]
        public void TestAggregate_FileNotInBasDirectory()
        {
            report.Reset(new DirectoryInfo("../../test_files/matcher"));
            FileInfo file = new FileInfo("../../test_files/conf/FileStatistics.xml");
            report.Aggregate(processor, file, result);
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// all arguments are valid, verify the aggregate result.
        /// </summary>
        [Test]
        public void TestAggregate1()
        {
            report.Reset(new DirectoryInfo("../../test_files"));

            // aggregate only one file
            FileInfo file = new FileInfo("../../test_files/processor/lineCounter/text1.txt");

            StatsResult result = new StatsResult();
            Metric metric = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result[metric] = 111;
            processor = new TextLineCounter("Text", new AnyFileMatcher());

            report.Aggregate(processor, file, result);

            // verify the aggregate result
            Assert.AreEqual(1, report.FileStats.AllKeys.Count,
                "Only one file aggregated, should contains the unique key.");
            Assert.AreEqual(111, report.FileStats["processor\\lineCounter\\text1.txt"][metric],
                "Aggregate failed for the FileStats not set correctly.");

            Assert.AreEqual(2, report.DirectoryStats.AllKeys.Count,
                "Three sub-directory is existed.");
            Assert.AreEqual(111, report.DirectoryStats["processor"][metric],
                "Aggregate failed for the DirectoryStats not set correctly.");
            Assert.AreEqual(111, report.DirectoryStats["processor\\lineCounter"][metric],
                "Aggregate failed for the DirectoryStats not set correctly.");

            Assert.AreEqual(1, report.AliasStats.AllKeys.Count,
                "Only one alias existed.");
            Assert.AreEqual(111, report.AliasStats["Text"][metric],
                "Aggregate failed for the AliasStatsnot set correctly.");

            Assert.AreEqual(111, report.OverallStats[metric],
                "Aggregate failed for the OverallStats set correctly.");
        }

        /// <summary>
        /// Test Aggregate(IFileProcessor processor, FileInfo file, StatsResult result),
        /// all arguments are valid, verify the aggregate result.
        /// </summary>
        [Test]
        public void TestAggregate2()
        {
            report.Reset(new DirectoryInfo("../../test_files"));

            // aggregate four files
            FileInfo file1 = new FileInfo("../../test_files/processor/lineCounter/text1.txt");
            FileInfo file2 = new FileInfo("../../test_files/processor/lineCounter/text2.txt");
            FileInfo file3 = new FileInfo("../../test_files/processor/lineCounter/text12.cs");
            FileInfo file4 = new FileInfo("../../test_files/matcher/type.java");

            StatsResult result1 = new StatsResult();
            Metric metric1 = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result1[metric1] = 10;

            StatsResult result2 = new StatsResult();
            Metric metric2 = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result2[metric2] = 20;

            StatsResult result3 = new StatsResult();
            Metric metric3 = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result3[metric3] = 30;

            StatsResult result4 = new StatsResult();
            Metric metric4 = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result4[metric4] = 40;

            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new TextLineCounter("Java", new ExtensionMatcher("java"));

            report.Aggregate(processor1, file1, result1);
            report.Aggregate(processor1, file2, result2);
            report.Aggregate(processor2, file3, result3);
            report.Aggregate(processor3, file4, result4);

            // verify the aggregate result
            Assert.AreEqual(4, report.FileStats.AllKeys.Count,
                "Only one file aggregated, should contains the unique key.");
            Assert.AreEqual(result1[metric1], report.FileStats["processor\\lineCounter\\text1.txt"][metric1],
                "Aggregate failed for the FileStats not set correctly.");
            Assert.AreEqual(result2[metric2], report.FileStats["processor\\lineCounter\\text2.txt"][metric2],
                "Aggregate failed for the FileStats not set correctly.");
            Assert.AreEqual(result3[metric3], report.FileStats["processor\\lineCounter\\text12.cs"][metric3],
                "Aggregate failed for the FileStats not set correctly.");
            Assert.AreEqual(result4[metric4], report.FileStats["matcher\\type.java"][metric4],
                "Aggregate failed for the FileStats not set correctly.");

            Assert.AreEqual(3, report.DirectoryStats.AllKeys.Count,
                "Three sub-directory is existed.");
            Assert.AreEqual(60, report.DirectoryStats["processor"][metric1],
                "Aggregate failed for the DirectoryStats not set correctly.");
            Assert.AreEqual(60, report.DirectoryStats["processor\\lineCounter"][metric2],
                "Aggregate failed for the DirectoryStats not set correctly.");
            Assert.AreEqual(40, report.DirectoryStats["matcher"][metric4],
                "Aggregate failed for the DirectoryStats not set correctly.");

            Assert.AreEqual(3, report.AliasStats.AllKeys.Count,
                "Three alias should be aggregaed.");
            Assert.AreEqual(30, report.AliasStats["Text"][metric1],
                "Aggregate failed for the AliasStatsnot set correctly.");
            Assert.AreEqual(30, report.AliasStats["CShape"][metric1],
                "Aggregate failed for the AliasStatsnot set correctly.");
            Assert.AreEqual(40, report.AliasStats["Java"][metric1],
                "Aggregate failed for the AliasStatsnot set correctly.");

            Assert.AreEqual(100, report.OverallStats[metric1],
                "Aggregate failed for the OverallStats set correctly.");
        }
    }
}