using System;
using System.IO;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This is the extended version of <see cref="FileStatistics"/> which supports
    /// some advanced operations including directory processing and reporting printing.
    /// The reason to separate this out is that we may not need to rely on the directory
    /// traversing algorithm incorporated here (which is recursive).
    ///
    /// <configuration>
    /// This class shares the same configuration logic as <see cref="FileStatistics"/>.
    /// The constructor simply propagates the namespace to the super class.
    /// </configuration>
    ///
    /// <thread-safety>
    /// All the operations are delegated. This class does not have any states and is fully
    /// dependant on <see cref="FileStatistics"/>' threading level.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class ExtendedFileStatistics : FileStatistics
    {
        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        public ExtendedFileStatistics()
        {
        }

        /// <summary>
        /// Constructor with namespace. Simply propagate to super constructor.
        /// </summary>
        ///
        /// <param name="nameSpace">the namespace to load config from</param>
        ///
        /// <exception cref="ArgumentNullException">if namespace is null</exception>
        /// <exception cref="ArgumentException">if namespace is empty after trimmed</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        public ExtendedFileStatistics(string nameSpace) : base(nameSpace)
        {
        }

        /// <summary>
        /// Process a directory in non-recursive mode.
        /// </summary>
        ///
        /// <param name="directory">the directory</param>
        ///
        /// <exception cref="ArgumentNullException">if directory is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        /// <exception cref="FileStatisticsException">if processor fails</exception>
        public void ProcessDirectory(DirectoryInfo directory)
        {
            ProcessDirectory(directory, false);
        }

        /// <summary>
        /// Process a directory in given mode.
        /// If it's non-recursive only the files in the directory is processed,
        /// otherwise every files in sub-directories are processed.
        /// </summary>
        ///
        /// <param name="directory">the directory</param>
        /// <param name="recursive">whether to process recursively</param>
        ///
        /// <exception cref="ArgumentNullException">if directory is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        /// <exception cref="FileStatisticsException">if processor fails</exception>
        public void ProcessDirectory(DirectoryInfo directory, bool recursive)
        {
            FileStatisticsHelper.CheckNull(directory, "directory");
            if (!directory.Exists)
            {
                throw new IOException("directory '" + directory.FullName + "' is not existed.");
            }

            try
            {
                // process files in directory
                FileInfo[] files = directory.GetFiles();
                for (int i =0; i < files.Length; ++i)
                {
                    this.ProcessFile(files[i]);
                }

                if (recursive)
                {
                    // if recursive is needed, process sub-directories
                    DirectoryInfo[] dirs = directory.GetDirectories();
                    for (int j = 0; j < dirs.Length; ++j)
                    {
                        ProcessDirectory(dirs[j], recursive);
                    }
                }
            }
            catch(FileStatisticsException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new FileStatisticsException("Failed to process files.", e);
            }
        }

        /// <summary>
        /// Print the report to console.
        /// </summary>
        public void PrintReport()
        {
            Console.Write(this.Reporting.GenerateReport());
        }

        /// <summary>
        /// Print the report to given file.
        /// </summary>
        ///
        /// <param name="file">the file to write to</param>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        /// <exception cref="StatsReportingException">if rendering fails</exception>
        public void PrintReport(FileInfo file)
        {
            FileStatisticsHelper.CheckNull(file, "file");
            try
            {
                string report = this.Reporting.GenerateReport();
                byte[] data = System.Text.Encoding.ASCII.GetBytes(report);

                // write the generated report
                FileStream write = file.OpenWrite();
                write.Write(data, 0, data.Length);
                write.Close();
            }
            catch(StatsReportingException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new IOException("Failed to print report.", e);
            }
        }
    }
}