using System;
using System.Collections;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// A internal help class for File Statistics component.
    /// This class defines some help functions.
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.1</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    internal class FileStatisticsHelper
    {
        /// <summary>
        /// Represents the const char '.'.
        /// </summary>
        public const char CHAR_DOT = '.';

        /// <summary>
        /// Private constructor prevent from being initialized.
        /// </summary>
        private FileStatisticsHelper()
        {
        }

        /// <summary>
        /// Check if the given obj is null, if it is null, throw <see cref="ArgumentNullException"/>.
        /// </summary>
        ///
        /// <exception cref="ArgumentNullException">if obj is null</exception>
        ///
        /// <param name="obj">the object to check.</param>
        /// <param name="name">the name used in <see cref="ArgumentNullException"/> to
        /// indicate the name of obj</param>
        public static void CheckNull(object obj, string name)
        {
            if (obj == null)
            {
                throw new ArgumentNullException(name, name + " should be non-null.");
            }
        }

        /// <summary>
        /// Check if the given str is a null or empty string.
        /// If it is null, throw <see cref="ArgumentNullException"/>.
        /// If it is empty string, throw <see cref="ArgumentException"/>.
        /// </summary>
        ///
        /// <exception cref="ArgumentNullException">if str is null</exception>
        /// <exception cref="ArgumentException">if str is an emtpy string</exception>
        ///
        /// <param name="str">the string to check</param>
        /// <param name="name">the name used in ArgumentException to indicate the name of str</param>
        public static void CheckNullOrEmptyString(string str, string name)
        {
            CheckNull(str, name);
            if (str.Trim().Length == 0)
            {
                throw new ArgumentException(name + " should be non-empty(trimmed) string.", name);
            }
        }

        /// <summary>
        /// Check the property value is valid. It the value is null or empty string,
        /// throw <see cref="ConfigurationException"/>.
        /// </summary>
        ///
        /// <param name="propertyValue">the value of property to check</param>
        /// <param name="propertyName">the name of property</param>
        ///
        /// <exception cref="ConfigurationException">if propertyValue is null or empty string.</exception>
        public static void CheckPropertyValue(string propertyValue, string propertyName)
        {
            if (propertyValue == null)
            {
                throw new ConfigurationException(
                    "The property value of " + propertyName + " should not be null.");
            }
            if (propertyValue.Trim().Length == 0)
            {
                throw new ConfigurationException("The property value of "
                    + propertyName + " should not be non-empty string.");
            }
        }

        /// <summary>
        /// Check if the given str is empty string or not.
        /// If it is empty string, throw <see cref="ArgumentException"/>.
        /// the str must be non-null.
        /// </summary>
        ///
        /// <exception cref="ArgumentException">if str is an emtpy string</exception>
        ///
        /// <param name="str">the string to check</param>
        /// <param name="name">the name used in ArgumentException to indicate the name of str</param>
        public static void CheckEmptyString(string str, string name)
        {
            if (str.Trim().Length == 0)
            {
                throw new ArgumentException(name + " should be non-empty(trimmed) string.", name);
            }
        }

        /// <summary>
        /// Check if the given list is non or empty list.
        /// If it is null, throw <see cref="ArgumentNullException"/>,
        /// if it is empty list, throw <see cref="ArgumentException"/>.
        /// </summary>
        ///
        /// <param name="list">the list to check</param>
        /// <param name="name">the name used in exception to indicate the name of list</param>
        public static void CheckEmptyList(IList list, string name)
        {
            CheckNull(list, name);
            if (list.Count == 0)
            {
                throw new ArgumentException("The list " + name + " should be non-empty.", name);
            }
        }

        /// <summary>
        /// Check toMatch is contained in string list.
        /// </summary>
        ///
        /// <param name="list">the string list.</param>
        /// <param name="toMatch">the string to match</param>
        /// <returns>true if found, otherwise false.</returns>
        public static bool Match(IList list, string toMatch)
        {
            foreach (string src in list)
            {
                if (src.Equals(toMatch))
                {
                    // found return true
                    return true;
                }
            }

            // no match found
            return false;
        }

        /// <summary>
        /// Save the souces into destionation and skip the overlapped source.
        /// </summary>
        ///
        /// <param name="destination">The destination list to save.</param>
        /// <param name="sources">The source list.</param>
        /// <param name="name">the name for exception message.</param>
        /// <param name="toLower">true if need to lower case.</param>
        ///
        /// <exception cref="ArgumentException">if sources is empty or contains non-String
        /// instance or contains empty string (after trimmed)</exception>
        public static void SaveListValue(IList destination, IList sources, string name, bool toLower)
        {
            foreach (object obj in sources)
            {
                // if the obj is null, throw ArgumentException
                if (obj == null)
                {
                    throw new ArgumentException(name + " contains null instance.", name);
                }
                // if the obj is not string type, throw ArgumentException
                if (!(obj is string))
                {

                    throw new ArgumentException(name + " contains non-string instance.", name);
                }

                // cast obj to string
                string source = (string) obj;

                // if the source is empty string, throw ArgumentException
                if (source.Trim().Length == 0)
                {
                    throw new ArgumentException(name + " contains emtpy string.", name);
                }
                if (toLower)
                {
                    source = source.ToLower();
                }

                // if already existed, just ignore it
                if (!destination.Contains(source))
                {
                    destination.Add(source);
                }
            }
        }

        /// <summary>
        /// Load config values for propertyName in nameSpace,and save the loaded values
        /// </summary>
        ///
        /// <param name="nameSpace">the namespace to load config from.</param>
        /// <param name="propertyName">the property to load config from.</param>
        /// <param name="values">the loaded values</param>
        /// <param name="toLower">true if need to lower case.</param>
        ///
        /// <exception cref="ConfigurationException">if property values does not contain any values
        /// or any value is empty after trimmed </exception>
        public static void LoadConfigValues(string nameSpace, string propertyName, IList values, bool toLower)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get the property values from the config file
                string[] propertyValues = cm.GetValues(nameSpace, propertyName);
                if (propertyValues.Length == 0)
                {
                    // the property contains no values
                    throw new ConfigurationException(
                        "The property (" + propertyName + ") doesn't hold any values.");
                }

                // add the type to types
                foreach (string propertyValue in propertyValues)
                {
                    // if value is empty string, throw ConfigurationException
                    if (propertyValue.Trim().Length == 0)
                    {
                        throw new ConfigurationException(
                            "There are empty string existed in value of " + propertyName);
                    }

                    string newValue = toLower ? propertyValue.ToLower() : propertyValue;

                    // if value already contained, ignore
                    if (!(values.Contains(newValue)))
                    {
                        values.Add(newValue);
                    }
                }
            }
            catch(ConfigurationException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new ConfigurationException(
                    "Failed to get the correct value for property (" + propertyName + ")", e);
            }
        }        
        
        /// <summary>
        /// Parse the string value to true or false.
        /// If failed to parse, throw ConfigurationException.
        /// </summary>
        ///
        /// <param name="sValue">the value to parse, must be low case</param>
        /// <param name="message">the message to show when the sValue can't be parsed.</param>
        /// <returns>true or false</returns>
        ///
        /// <exception cref="ConfigurationException">if failed to parse</exception>
        public static bool CheckBoolValue(string sValue, string message)
        {
            // this literal is just used here, so not defined as class const
            string[] yes = new string[] {"yes", "true", "on"};
            string[] no = new string[] {"no", "false", "off"};

            // compare with the yes string[], if match, return true
            for (int i = 0; i < yes.Length; ++i)
            {
                if (sValue.Equals(yes[i]))
                {
                    return true;
                }
            }

            // compare with the no string[], if match, return true
            for (int i = 0; i < no.Length; ++i)
            {
                if (sValue.Equals(no[i]))
                {
                    return false;
                }
            }

            // can't parse, throw ConfigurationException
            throw new ConfigurationException(message);
        }
        
        /// <summary>
        /// Check the key is existed in arguments, and the 
        /// value of it is string type.
        /// </summary>
        /// 
        /// <param name="arguments">the arguments dictionary</param>
        /// <param name="key">the property key to check</param>
        /// <returns>the argument value</returns>
        /// 
        /// <exception cref="ArgumentException">
        /// if the value mapped with the key is not string type</exception>
        public static bool CheckArgument(IDictionary arguments, string key)
        {
            object argument = arguments[key];
            if (argument == null)
            {
                return false;
            }
            else 
            {
                if (argument is string) {
                    return true;
                }
                else
                {
                    throw new ArgumentException(
                        key + " does not correspond to a String");
                }
            }
        }

        /// <summary>
        /// Create instance use reflection, and the construct of take two string parameter.
        /// </summary>
        ///
        /// <param name="className">The class type of the instance to created.</param>
        /// <param name="arguments">the parameters of the constructor.</param>
        /// <param name="assemblyName">The assembly name, if it is null, use the current assembly.</param>
        /// <returns>the created instance.</returns>
        ///
        /// <exception cref="Exception">Exception throwed when constructs.</exception>
        public static object CreateInsance(string className, string[] arguments, string assemblyName)
        {
            if (assemblyName == null)
            {
                // assemblyName is null, use current assembly.
                assemblyName = Assembly.GetCallingAssembly().GetName().Name;
            }

            // if failed to create instance, just throw the excetpion
            return AppDomain.CurrentDomain.CreateInstanceAndUnwrap(
                assemblyName, className, false, BindingFlags.Default, null,
                arguments, null, null, null);
        }
    }
}
