using System;
using System.Collections;
using TopCoder.Unix.MagicNumbers;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileTypeMatcher"/> concrete implementation that matches file types with
    /// one or more type names (as configured in Magic Numbers).
    ///
    /// <configuration>
    /// This class can be constructed via reflection by the constructor contract of
    /// <see cref="IFileMatcher"/>.
    /// The supported property is "types" which is potentially multi-valued.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class TypeNameMatcher : FileTypeMatcher
    {
        /// <summary>
        /// Configuration key that contains multiple file type names to match.
        /// Empty file names are not allowed even if they are valid in the Magic Numbers component.
        /// </summary>
        public static readonly string TypesPropertyKey = "types";

        /// <summary>
        /// A list of file name types (string) to match.
        /// This list is initialized in constructor and is immutable.
        /// </summary>
        private IList types = new ArrayList();

        /// <summary>
        /// Constructor with single file type name.
        /// </summary>
        ///
        /// <param name="type">the file type name</param>
        ///
        /// <exception cref="ArgumentNullException">if type is null</exception>
        /// <exception cref="ArgumentException">if type is empty</exception>
        public TypeNameMatcher(string type)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(type, "type");

            types.Add(type);
        }

        /// <summary>
        /// Constructor with multiple file type names. Simply save a clone of the list.
        /// </summary>
        ///
        /// <param name="types">the file type names to match</param>
        ///
        /// <exception cref="ArgumentNullException">if types is null</exception>
        /// <exception cref="ArgumentException">if types is empty or contains non-String
        /// instance or contains empty string (after trimmed)</exception>
        public TypeNameMatcher(IList types)
        {
            // verify the types is valid
            FileStatisticsHelper.CheckEmptyList(types, "types");

            // save the types
            FileStatisticsHelper.SaveListValue(this.types, types, "types", false);
        }

        /// <summary>
        /// Constructor that loads type names from a property.
        /// It should look for an "types" sub-property and extract multiple values.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if property values does not contain any values
        /// or any value is empty after trimmed </exception>
        public TypeNameMatcher(string nameSpace, string prefix)
        {
            // verify the namespace is null or empty string
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            // get the property name
            string propertyName = (prefix == null) ?
                TypesPropertyKey : prefix + TypesPropertyKey;

            // load the config values
            FileStatisticsHelper.LoadConfigValues(nameSpace, propertyName, this.types, false);
        }

        /// <summary>
        /// Check whether any of the type names match the type. The matching is case-sensitive.
        /// </summary>
        ///
        /// <param name="type">the type to match</param>
        /// <returns>whether the type matches</returns>
        protected override bool Match(FileType type)
        {
            return FileStatisticsHelper.Match(this.types, type.Name);
        }
    }
}
