using System;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Metric represents a measurement of the <see cref="StatsResult"/>. A metric is identified by its name.
    /// Metrics with the same name will be aggregated. This class overrides Equals() and GetHashCode()
    /// so that it can be used as map keys.
    ///
    /// <thread-safety>
    /// This class is immutable and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class Metric
    {
        /// <summary>
        /// The name of the metric. Acts as an identifier.
        /// </summary>
        private string name;

        /// <summary>
        /// The display name in the rendered result. If this is used in xml reporting the displayn name
        /// should be well formed element name.
        /// </summary>
        private string displayName;

        /// <summary>
        /// A description for the metric. This does not make a significance in this release.
        /// </summary>
        private string description;

        /// <summary>
        /// Get the name of the metric.
        /// </summary>
        ///
        /// <value>Represents the name of the metric.</value>
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        /// <summary>
        /// Get the display name of the metric.
        /// </summary>
        ///
        /// <value>Represents the display name of the metric.</value>
        public string DisplayName
        {
            get
            {
                return this.displayName;
            }
        }

        /// <summary>
        /// Get the description of the metric.
        /// </summary>
        ///
        /// <value>Represents the description of the metric.</value>
        public string Description
        {
            get
            {
                return this.description;
            }
        }

        /// <summary>
        /// Constructor will all arguments initialized.
        /// </summary>
        ///
        /// <param name="name">the name of the metric</param>
        /// <param name="displayName">the display name of the metric</param>
        /// <param name="description">the description of the metric</param>
        ///
        /// <exception cref="ArgumentNullException">if any argument is null</exception>
        /// <exception cref="ArgumentException">if any argument is empty after trimmed</exception>
        public Metric(string name, string displayName, string description)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(name, "name");
            FileStatisticsHelper.CheckNullOrEmptyString(displayName, "displayName");
            FileStatisticsHelper.CheckNullOrEmptyString(description, "description");

            // save the arguments to variables
            this.name = name;
            this.displayName = displayName;
            this.description = description;
        }

        /// <summary>
        /// Equals contract that is based on name.
        /// </summary>
        ///
        /// <param name="obj">the object to check</param>
        public override bool Equals(object obj)
        {
            if (typeof(Metric).IsAssignableFrom(obj.GetType()))
            {
                // compare the name of the Metric with case-insensitive.
                return Name.ToLower().Equals(((Metric)obj).Name.ToLower());
            }
            else
            {
                // if obj not Metric type, return false
                return false;
            }
        }

        /// <summary>
        /// hashCode contract that is based on name.
        /// </summary>
        ///
        /// <returns>the hash code.</returns>
        public override int GetHashCode()
        {
            // return the name's hashCode
            return Name.GetHashCode();
        }
    }
}