using System;
using System.IO;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Unit test for <see cref="TextLineCounter"/> class.
    /// It mainly tests the following:
    /// static member:Name, DisplayName, IgnoreBlankLinePropertyKey
    /// constructor: TextLineCounter(string, IFileMatcher), TextLineCounter(string, string)
    /// property: SupportedMetrics
    /// member: Process(FileInfo)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class TextLineCounterUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="IFileMatcher"/>, used to construct the processor.
        /// </summary>
        private IFileMatcher matcher = new AnyFileMatcher();

        /// <summary>
        /// Represents an alias  for the processor, used to construct the processor.
        /// </summary>
        private string alias = "Text";

        /// <summary>
        /// Represents an instance of <see cref="FileProcessorBase"/>, used to test it's public member.
        /// </summary>
        private IFileProcessor processor;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);

            processor = new TextLineCounter(alias, matcher);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Test static member Name, it should be 'Line Count'.
        /// </summary>
        [Test]
        public void TestName()
        {
            Assert.AreEqual("Line Count", TextLineCounter.Name,
                "Name of TextLineCounter should be 'Line Count'");
        }

        /// <summary>
        /// Test static member DisplayName, it should be 'line'.
        /// </summary>
        [Test]
        public void TestDisplayName()
        {
            Assert.AreEqual("line", TextLineCounter.DisplayName,
                "Name of TextLineCounter should be 'line'");
        }
        
        /// <summary>
        /// Test static member IgnoreBlankLinePropertyKey, it should be 'ignore_blank_line'.
        /// </summary>
        [Test]
        public void TestIgnoreBlankLinePropertyKey()
        {
            Assert.AreEqual("line", TextLineCounter.IgnoreBlankLinePropertyKey,
                "IgnoreBlankLinePropertyKey of TextLineCounter should be 'ignore_blank_line'");
        }

        /// <summary>
        /// Test property SupportedMetrics Getter, only one metric is supported
        /// </summary>
        public void TestSupportedMetrics_Getter()
        {
            Assert.AreEqual(1, processor.SupportedMetrics.Count,
                "Only one metric is supported.");

            Assert.AreEqual(TextLineCounter.Name, ((Metric) processor.SupportedMetrics[0]).Name,
                "Failed to set the correct name of Supported metric");
            Assert.AreEqual(TextLineCounter.DisplayName, ((Metric) processor.SupportedMetrics[0]).DisplayName,
                "Failed to set the correct display name of Supported metric");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher), with alias is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_AliasIsNull()
        {
            new TextLineCounter(null, matcher);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher), with alias is emtpy,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor1_AliasIsEmpty()
        {
            new TextLineCounter("  ", matcher);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher), with matcher is null
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_MatcherIsNull()
        {
            new TextLineCounter(alias, (IFileMatcher) null);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher), with both arguments are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(processor, "Failed to create instance of TextLineCounter.");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NamespaceIsNull()
        {
            new TextLineCounter(null, "processor1");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with nameSpace is emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NamespaceIsEmpty()
        {
            new TextLineCounter("  ", "processor1");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with prefix is null,
        /// AEE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PrefixIsNull()
        {
            new TextLineCounter(UnitTestHelper.DEFAULT_NAMESPACE, (string) null);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor1_ConfigFileError1()
        {
            new TextLineCounter(UnitTestHelper.INVALID_NAMESPACE, "Processor1");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError2()
        {
            new TextLineCounter(UnitTestHelper.INVALID_NAMESPACE, "Processor2");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError3()
        {
            new TextLineCounter(UnitTestHelper.INVALID_NAMESPACE, "Processor3");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError4()
        {
            new TextLineCounter(UnitTestHelper.INVALID_NAMESPACE, "Processor4.");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with ignore_blank_line is invalid,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError5()
        {
            new TextLineCounter(UnitTestHelper.INVALID_NAMESPACE, "Processor20.");
        }

        /// <summary>
        /// Test ctor TextLineCounter(string nameSpace, string prefix), with both argument are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            processor = new TextLineCounter(UnitTestHelper.DEFAULT_NAMESPACE, "Processor1.");
            Assert.IsNotNull(processor, "Failed to create instance of TextLineCounter.");
        }
        
        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine),
        /// with alias is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_AliasIsNull()
        {
            new TextLineCounter(null, matcher, false);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine),
        /// with alias is emtpy, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor3_AliasIsEmpty()
        {
            new TextLineCounter("  ", matcher, false);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine),
        /// with matcher is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_MatcherIsNull()
        {
            new TextLineCounter(alias, (IFileMatcher) null);
        }

        /// <summary>
        /// Test ctor TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine),
        /// with all arguments are valid, instance should be created.
        /// </summary>
        [Test]
        public void TestCtor3()
        {
            processor = new TextLineCounter(alias, matcher, false);
            Assert.IsNotNull(processor, "Failed to create instance of TextLineCounter.");
        }

        /// <summary>
        /// Test Process(FileInfo file), with file is null, ANE is epxected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestProcess_FileIsNull()
        {
            processor.Process(null);
        }

        /// <summary>
        /// Test Process(FileInfo file), with file is not existed, IOException is epxected.
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestProcess_FileNotExist()
        {
            processor.Process(new FileInfo("../test_files/NotExisted."));
        }

        /// <summary>
        /// Test Process(FileInfo file), the returned result should contains the correct line count.
        /// </summary>
        [Test]
        public void TestProcess1()
        {
            verifyProcess("../../test_files/Processor/LineCounter/text1.txt", 0);
        }

        /// <summary>
        /// Test Process(FileInfo file), the returned result should contains the correct line count.
        /// </summary>
        [Test]
        public void TestProcess2()
        {
            verifyProcess("../../test_files/Processor/LineCounter/text2.txt", 1);
        }

        /// <summary>
        /// Test Process(FileInfo file), the returned result should contains the correct line count.
        /// </summary>
        [Test]
        public void TestProcess3()
        {
            verifyProcess("../../test_files/Processor/LineCounter/text3.txt", 105340);
        }


        /// <summary>
        /// Test Process(FileInfo file), the returned result should contains the correct line count.
        /// </summary>
        [Test]
        public void TestProcess4()
        {
            verifyProcess("../../test_files/Processor/LineCounter/text4.txt", 1);
        }

        /// <summary>
        /// Test Process(FileInfo file), the returned result should contains the correct line count.
        /// </summary>
        [Test]
        public void TestProcess5()
        {
            verifyProcess("../../test_files/Processor/LineCounter/text5.txt", 20);
        }

        /// <summary>
        /// Test the process, and verify the returned result with expected.
        /// </summary>
        ///
        /// <param name="file">the file</param>
        /// <param name="expected">the metric value expected.</param>
        private void verifyProcess(string file, long expected)
        {
            StatsResult result = processor.Process(new FileInfo(file));
            Assert.AreEqual(expected, result[(Metric) result.Metrics[0]],
                "Failed to get the correct line count.");
        }
    }
}
