using System;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Unit Test for <see cref="StatsResult"/> class.
    /// It mainly tests the following:
    /// constructor: StatsResult(), StatsResult(StatsResult)
    /// property: Metrics
    /// member: indexer, Aggregate(StatsResult)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class StatsResultUnitTest
    {
        /// <summary>
        /// Represents a instance of <see cref="StatsResult"/>, used to test it's members.
        /// </summary>
        private StatsResult statsResult;

        /// <summary>
        /// Set up for each test,
        /// construct an instance of Metric.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            statsResult = new StatsResult();
        }

        /// <summary>
        /// Test ctor StatsResult(), instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(statsResult, "Failed to create instance of StatsResult.");
        }

        /// <summary>
        /// Test ctor StatsResult(StatsResult result), with result is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_ResultIsNull()
        {
            StatsResult result = new StatsResult(null);
        }

        /// <summary>
        /// Test ctor StatsResult(StatsResult result), with result is valid,
        /// a clone instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2_Empty()
        {
            StatsResult result = new StatsResult(statsResult);
            Assert.IsNotNull(result, "Failed to create instance of StatsResult.");
            Assert.AreEqual(0, result.Metrics.Count,
                "Failed to create the correct instance of StatsResult.");
        }

        /// <summary>
        /// Test ctor StatsResult(StatsResult result), with result is valid,
        /// a clone instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Metric metric = new Metric("Unit Test", "Unit", "Test");
            statsResult[metric] = 11;
            StatsResult result = new StatsResult(statsResult);
            Assert.IsNotNull(result, "Failed to create instance of StatsResult.");
            Assert.AreEqual(1, result.Metrics.Count,
                "Failed to create the correct instance of StatsResult.");
            Assert.AreEqual(metric, result.Metrics[0],
                "Failed to clone the correct StatsResult");
            Assert.AreEqual(11, result[metric],
                "Failed to clone the correct StatsResult");
        }

        /// <summary>
        /// Test property Metrics Getter, with statsResult contains no metric, it should return empty list.
        /// </summary>
        public void TestMetrics_Getter_Empty()
        {
            StatsResult result = new StatsResult();
            Assert.AreEqual(0, result.Metrics.Count, "Failed to get the correct metric list.");
        }

        /// <summary>
        /// Test property Metrics Getter, it should return the list of all the associated metrics.
        /// </summary>
        public void TestMetrics_Getter()
        {
            int count = 10;
            Metric[] metrics = new Metric[count];

            // add some metrics for test
            for (int i = 0; i < count; ++i)
            {
                metrics[i] = new Metric(i.ToString(), i.ToString(), i.ToString());
                statsResult[metrics[i]] = i;
            }

            // get the metric list
            IList results = statsResult.Metrics;

            // add another metric
            Metric m = new Metric(count.ToString(), count.ToString(), count.ToString());
            statsResult[m] = count;

            // verify the returned metrics
            // the returned should be cloned
            Assert.AreEqual(count, results.Count, "Failed to get the correct list.");
            for (int i = 0; i < count; ++i)
            {
                Assert.IsTrue(results.Contains(metrics[i]), "Failed to get the correct metric list.");
            }
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Getter, with metric is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexer_Getter_MetriIsNull()
        {
            long quantity = statsResult[null];
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Getter, with metric not existed, AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestIndexer_Getter_MetricNotExist()
        {
            Metric metric = new Metric("1", "1", "1");
            long quantity = statsResult[metric];
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Getter, it should return the quantity number with
        /// the specified metric.
        /// </summary>
        [Test]
        public void TestIndexer_Getter()
        {
            Metric metric = new Metric("1", "1", "1");
            statsResult[metric] = 111;

            Assert.AreEqual(111, statsResult[metric],
                "Failed to get the correct quantity number.");
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Setter, with metric is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexer_Setter_MetricIsNull()
        {
            statsResult[null] = 1;
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Setter, add new metric,
        /// </summary>
        [Test]
        public void TestIndexer_Setter_Add()
        {
            Metric metric = new Metric("1", "1", "1");
            statsResult[metric] = 111;

            Assert.AreEqual(111, statsResult[metric],
                "Failed to get the correct quantity number.");
        }

        /// <summary>
        /// Test Indexer this[Metric metric] Setter, add new metric,
        /// </summary>
        [Test]
        public void TestIndexer_Setter_Modify()
        {
            Metric metric = new Metric("1", "1", "1");
            statsResult[metric] = 111;
            statsResult[metric] = 555;

            Assert.AreEqual(555, statsResult[metric],
                "Failed to set the quantity number to new value.");
        }

        /// <summary>
        /// Test Aggregate(StatsResult result), result is null, ANE is expected
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregate_ResultIsNull()
        {
            statsResult.Aggregate(null);
        }

        /// <summary>
        /// Test  Aggregate(StatsResult result), with result don't contain any metric.
        /// </summary>
        [Test]
        public void TestAggregate_ResultIsEmpty()
        {
            StatsResult result = new StatsResult();
            statsResult.Aggregate(result);
            Assert.AreEqual(0, statsResult.Metrics.Count,
                "Failed to aggreagte the statsResult");
        }

        /// <summary>
        /// Test ctor StatsResult(StatsResult result), result contans metric.
        /// </summary>
        [Test]
        public void TestAggregate1()
        {
            Metric metric = new Metric("Unit Test", "Unit", "Test");
            statsResult[metric] = 11;

            StatsResult result = new StatsResult();
            result.Aggregate(statsResult);

            Assert.AreEqual(1, result.Metrics.Count,
                "Failed to create the correct instance of StatsResult.");
            Assert.AreEqual(metric, result.Metrics[0],
                "Failed to clone the correct StatsResult");
            Assert.AreEqual(11, result[metric],
                "Failed to clone the correct StatsResult");
        }


        /// <summary>
        /// Test ctor StatsResult(StatsResult result), result contans metric.
        /// </summary>
        [Test]
        public void TestAggregate2()
        {
            Metric metric = new Metric("Unit Test", "Unit", "Test");
            statsResult[metric] = 11;

            StatsResult result = new StatsResult();
            result.Aggregate(statsResult);
            result.Aggregate(statsResult);

            Assert.AreEqual(1, result.Metrics.Count,
                "Failed to create the correct instance of StatsResult.");
            Assert.AreEqual(metric, result.Metrics[0],
                "Failed to clone the correct StatsResult");
            Assert.AreEqual(22, result[metric],
                "Failed to clone the correct StatsResult");
        }
    }
}